/*
        Thymio-II Bootloader

        Copyright (C) 2011 Philippe Retornaz <philippe dot retornaz at epfl dot ch>,
        Mobots group (http://mobots.epfl.ch), Robotics system laboratory (http://lsro.epfl.ch)
        EPFL Ecole polytechnique federale de Lausanne (http://www.epfl.ch)

        See authors.txt for more details about other contributors.

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU Lesser General Public License as published
        by the Free Software Foundation, version 3 of the License.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public License
        along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

.include "p24fxxxx.inc"

.equ FLASH_PAGE_INSTRUCTION_COUNT, 512
.equ FLASH_ROW_INSTRUCTION_COUNT, 64
.equ FLASH_ERASE_PAGE_CODE, 0x4042
.equ FLASH_PROG_ROW_CODE, 0x4001


.text 
.section .text,code

; protect against non-programmed pic.
reset

.global _do_flash_page
; arg 0: page_hiptr
; arg 1: page_lowptr
; arg 2: page_buf
_do_flash_page:
	push w8
	push w9
	push w10
	push w11
	push w12

	mov w2, w5
	mov w1, w6
	; Erase the page
	
	mov w0, TBLPAG 
	
	; Setup flash for erase operation
	mov #FLASH_ERASE_PAGE_CODE, w1
	mov w1, NVMCON
	
	; Dummy write to select the page to erase
	tblwtl w6, [w6]
	
	; Do the real flash page erase
	rcall do_key_seq

	; Prepare to write rows
	
	; Setup flash for row write
	mov #FLASH_PROG_ROW_CODE, w0
	mov w0, NVMCON
	
	; Setup pointers
	mov #(FLASH_PAGE_INSTRUCTION_COUNT / FLASH_ROW_INSTRUCTION_COUNT), w4


dfp_prog_loop_page:

	; Row instruction counter
	mov #FLASH_ROW_INSTRUCTION_COUNT, w3
	
dfp_prog_loop_row:
	; Write into flash from buffer
	tblwtl [w5++], [w6]
	tblwth [w5++], [w6++]

	; Decrement and check row counter
	dec w3, w3
	bra NZ, dfp_prog_loop_row

	; Do the real flash write
	rcall do_key_seq
	
	; Decrement and check page counter
	dec w4, w4
	bra NZ, dfp_prog_loop_page
	
; now verify the whole thing
; reset both pointer to their original values
	mov #2048, w0
	sub w5, w0, w5
	mov #1024, w0
	sub w6, w0, w0

	mov #FLASH_PAGE_INSTRUCTION_COUNT, w4

dfp_verify_loop:
	; Compare low word
	mov [w5++], w1
	tblrdl [w0], w2
	cpseq w1, w2
	bra dfp_error_prog_1
	
	; Compare high byte
	mov [w5++], w1
	tblrdh [w0++], w2
	cpseq w1, w2
	bra dfp_error_prog_2
	
	; Decrement and check instruction counter
	dec w4, w4
	bra NZ, dfp_verify_loop
		
	mov #0, w0
	bra dfp_exit
	
dfp_error_prog_1:
	inc w0, w0
dfp_error_prog_2:

dfp_exit:
	pop w12
	pop w11
	pop w10
	pop w9
	pop w8
	
	return


; Do the key sequence and start the flash programming
; in: nothing
; out: nothin
; modify: w0,w1
do_key_seq:
	disi #6
	mov #0x55, w0
	mov w0, NVMKEY
	mov #0xaa, w1
	mov w1, NVMKEY
	bset NVMCON, #WR
	nop
	nop
	btsc NVMCON, #15
	bra	$-2
	return

.global _read_page
_read_page:
; arg 0: page_hiptr
; arg 1: page_lowptr
; arg 2: page_buf
	mov w0, TBLPAG 
	mov #512, w3
	
read_loop:
	tblrdl [w1], [w2++]
	tblrdh [w1++], [w2++]
	dec w3, w3
	bra nz,read_loop
	return
